import logging
import yupi

try:
    import traja
except (ModuleNotFoundError, ImportError):
    logging.error(f'''Fail at importing 'traja' library

You can refer to https://github.com/traja-team/traja#installation-and-setup''')


def yupi2traja(yupi_trajectory: yupi.Trajectory) -> traja.TrajaDataFrame:
    """Converts a yupi trajectory to a traja dataframe.

    Parameters
    ----------
    yupi_trajectory : yupi.Trajectory
        Yupi trajectory input.

    Returns
    -------
    traja.TrajaDataFrame
        Resulting traja dataframe.

    Raises
    ------
    ValueError
        If the yupi trajectory dimension is not equal 2.
    """

    if yupi_trajectory.dim != 2:
        raise ValueError("Trajectory object has incompatible dimensions with traja")
    
    x = yupi_trajectory.r.x
    y = yupi_trajectory.r.y
    t = yupi_trajectory.t
    return traja.TrajaDataFrame({"x": x, "y": y, "time": t})

def traja2yupi(traja_trajectory: traja.TrajaDataFrame) -> yupi.Trajectory:
    """Converts a traja dataframe to a yupi trajectory.

    Parameters
    ----------
    traja_trajectory : traja.TrajaDataFrame
        Traja dataframe input.

    Returns
    -------
    yupi.Trajectory
        Resulting yupi trajectory.
    """

    x = traja_trajectory.x.to_numpy()
    y = traja_trajectory.y.to_numpy()
    t = traja_trajectory.time.to_numpy()
    return yupi.Trajectory(x=x, y=y, t=t)


from typing import Iterable
import numpy as np
from typing import Iterable
from datetime import datetime, timedelta

def seconds2datetime(time: np.ndarray, init_datetime=None) -> Iterable[datetime]:
    """Converts a time array of seconds to a datetime iterable

    Parameters
    ----------
    time : np.ndarray
        Time array input.
    init_datetime : datetime, optional
        Initial datetime, by default None.

        If None, then the initial datetime is taken as ``datetime.now()``.

    Returns
    -------
    Iterable[datetime]
        Resulting datetime iterable.
    """
 
    if init_datetime is None:
        init_datetime = datetime.now()
    return [init_datetime + timedelta(seconds=s) for s in time]


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yupiwrap']

package_data = \
{'': ['*']}

install_requires = \
['tracktable>=1.5.0,<2.0.0', 'traja>=0.2.8,<0.3.0', 'yupi>=0.8.0,<0.9.0']

setup_kwargs = {
    'name': 'yupiwrap',
    'version': '0.1.2',
    'description': '',
    'long_description': '# yupiwrap\n\nThis repository contains functions to simplify the conversion of Trajectory data\namong [yupi](https://yupi.readthedocs.io/en/latest/) and other useful software libraries designed for analyzing trajectories.\n\nStanding for *Yet Underused Path Instruments*, [yupi](https://yupi.readthedocs.io/en/latest/) is a set of tools designed for collecting, generating and processing trajectory data. The structure of yupi aims to standardize the usage and storage of general purpose trajectories independently of its dimensions. We believe it is useful to be able to convert, when possible, yupi trajectories to the data structures used by other libraries to\nempower our users with the tools offered by third parties. With the same spirit, we offer the possibility of converting data from other libraries to yupi trajectories.\n\n## Installation\n\nCurrent recommended installation method is via the pypi package:\n\n```cmd\npip install yupiwrap\n```\n\nIt will install required dependencies such as [yupi package](https://pypi.org/project/yupi/) from pypi.\n\n## Compatible libraries\n\n### traja\n\nThe [Traja Python package](https://traja.readthedocs.io/en/latest/index.html) is a toolkit for the numerical characterization and analysis of the trajectories of moving animals. It provides several machine learning tools that are not yet implemented in yupi. Even when it is limited to two-dimensional trajectories, there are many resources that traja can offer when dealing with 2D Trajectories in [yupi](https://yupi.readthedocs.io/en/latest/).\n\n#### Converting a *yupi.Trajectory* into a *traja DataFrame*\n\nLet\'s create a trajectory with yupi:\n\n```python\nfrom yupi import Trajectory\n\nx = [0, 1.0, 0.63, -0.37, -1.24, -1.5, -1.08, -0.19, 0.82, 1.63, 1.99, 1.85]\ny = [0, 0, 0.98, 1.24, 0.69, -0.3, -1.23, -1.72, -1.63, -1.01, -0.06, 0.94]\n\ntrack = Trajectory(x=x, y=y, traj_id="Spiral")\n```\n\nWe can convert it to a traja DataFrame simply by:\n\n```python\nfrom yupiwrap import yupi2traja\n\ntraja_track = yupi2traja(track)\n```\n\n⚠️ Only *yupi.Trajectory* objects with two dimensions can be converted to *traja DataFrame* due to traja limitations.\n\n#### Converting a *traja DataFrame* into a *yupi.Trajectory*\n\nIf you have a *traja DataFrame* you can always convert it to a *yupi.Trajectory* by using:\n\n```python\nfrom yupiwrap import traja2yupi\n\nyupi_track = traja2yupi(traja_track)\n```\n\n### tracktable\n\n[Tracktable](https://github.com/sandialabs/tracktable) provides a set of tools for handling 2D and 3D trajectories as well as Terrain trajectories. The core data structures and algorithms on this package are implemented in C++ for speed and more efficient memory use.\n\n#### Converting a *yupi.Trajectory* into a tracktable trajectory\n\nLet\'s create a trajectory with yupi:\n\n```python\nfrom yupiwrap.tracktable import yupi2tracktable, tracktable2yupi\nfrom yupi import Trajectory\n\n# Creating a yupi trajectory representing terrain coordinates\npoints = [[-82.359415, 23.135012],[-82.382116, 23.136252]]\ntrack_1 = Trajectory(points=points, traj_id="ter_track")\n\n# Creating a 2D yupi trajectory\npoints = [[0, 0], [1.0, 0], [0.63, 0.98], [-0.37, 1.24], [-1.24, 0.69],\n          [-1.5, -0.3], [-1.08, -1.23], [-0.19, -1.72], [0.82, -1.63],\n          [1.63, -1.01], [1.99, -0.06], [1.85, 0.94]]\ntrack_2 = Trajectory(points=points, traj_id="2d_track")\n\n# Creating a 3D yupi trajectory\npoints = [[0,0,0], [1,1,3], [3,2,5]]\ntrack_3 = Trajectory(points=points, traj_id="3d_track")\n```\n\nWe can convert these tracks to tracktable trajectories simply by:\n\n```python\ntracktable_track_1 = yupi2tracktable(track_1, is_terrestrial=True)\ntracktable_track_2 = yupi2tracktable(track_2)\ntracktable_track_3 = yupi2tracktable(track_3)\n```\n\n⚠️ If a 3D yupi trajectory is converted to a tracktable trajectory with `is_terrestrial=True` then the `z` axis values are stored as a property called `\'altitude\'` for each point.\n\n⚠️ Only *yupi.Trajectory* objects with two or three dimensions can be converted to tracktable trajectories due to tracktable limitations.\n\n#### Converting a tracktable trajectory into a *yupi.Trajectory*\n\nIf you have a tracktable trajectory you can always convert it to a *yupi.Trajectory* by using:\n\n```python\n# Converting the trajectory from tracktable to yupi\nyupi_track_1 = tracktable2yupi(tracktable_track_1)\n```\n',
    'author': 'Gustavo Viera-López',
    'author_email': 'gvieralopez@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime
import logging
import numpy as np
from typing import Callable, Iterable, Union
from yupiwrap.utils import seconds2datetime
import yupi

try:
    import tracktable.domain.terrestrial as tdt_ter
    import tracktable.domain.cartesian2d as tdt_2d
    import tracktable.domain.cartesian3d as tdt_3d
except (ModuleNotFoundError, ImportError):
    logging.error(f'''Fail at importing 'tracktable' library

You can refer to https://github.com/sandialabs/tracktable#getting-tracktable''')

TracktableTrajectory = Union[
    tdt_ter.Trajectory,
    tdt_2d.Trajectory,
    tdt_3d.Trajectory
]

def yupi2tracktable(yupi_trajectory: yupi.Trajectory,
                    time_conv: Callable[[np.ndarray], Iterable] = seconds2datetime,
                    init_datetime: datetime = None,
                    is_terrestrial = False) -> TracktableTrajectory:
    """Converts a yupi trajectory to a tracktable trajectory.

    Parameters
    ----------
    yupi_trajectory : yupi.Trajectory
        Yupi trajectory input.
    time_conv : Callable[[np.ndarray], Iterable], optional
        This function translates theyupi trajectory time array to a
        datetime iterable, by default ``yupiwrap.utils.seconds2datetime``.
    init_datetime : datetime, optional
        Initial datetime taken as reference for the construction of the
        datetime iterable, by default None.
    is_terrestrial : bool, optional
        If True the returned tracktable trajectory will be of terrestial
        type instead of the default cartesian type, by default False.

    Returns
    -------
    TracktableTrajectory
        The result tracktable trajectory.

    Raises
    ------
    ValueError
        If the yupi trajectory dimension is not 2 or 3.
    """

    dates = time_conv(yupi_trajectory.t, init_datetime)
    track_points = []
    if yupi_trajectory.dim == 2:
        for tp, date in zip(yupi_trajectory, dates):
            if is_terrestrial:
                point = tdt_ter.TrajectoryPoint(tp.r[0], tp.r[1])
            else:
                point = tdt_2d.TrajectoryPoint(tp.r[0], tp.r[1])
            point.object_id = yupi_trajectory.id
            point.timestamp = date
            track_points.append(point)
        if is_terrestrial:
            return tdt_ter.Trajectory.from_position_list(track_points)
        else:
            return tdt_2d.Trajectory.from_position_list(track_points)
    elif yupi_trajectory.dim == 3:
        for tp, date in zip(yupi_trajectory, dates):
            if is_terrestrial:
                point = tdt_ter.TrajectoryPoint(tp.r[0], tp.r[1])
                point.properties['altitude'] = tp.r[2]
            else:
                point = tdt_3d.TrajectoryPoint(tp.r[0], tp.r[1], tp.r[2])
            point.object_id = yupi_trajectory.id
            point.timestamp = date
            track_points.append(point)
        if is_terrestrial:
            return tdt_ter.Trajectory.from_position_list(track_points)
        else:
            return tdt_3d.Trajectory.from_position_list(track_points)
    else:
        raise ValueError("Trajectory object has incompatible dimensions with tracktable")

def tracktable2yupi(tracktable_trajectory: TracktableTrajectory) -> yupi.Trajectory:
    """Converts a tracktable trajectory to a yupi trajectory

    Parameters
    ----------
    tracktable_trajectory : TracktableTrajectory
        Tracktable trajectory input.

    Returns
    -------
    yupi.Trajectory
        Resulting yupi trajectory.

    Raises
    ------
    ValueError
        If the input is not a valid tracktable trajectory
    """

    is_ter = isinstance(tracktable_trajectory, tdt_ter.Trajectory)
    is_2d = isinstance(tracktable_trajectory, tdt_2d.Trajectory)
    is_3d = isinstance(tracktable_trajectory, tdt_3d.Trajectory)

    if is_ter + is_2d + is_3d != 1:
        raise ValueError('Invalid tracktable trajectory')

    points, t = [], []
    init_date = None
    for i, point in enumerate(tracktable_trajectory):
        if i == 0:
            init_date: datetime = point.timestamp
        current_date: datetime = point.timestamp
        if is_ter:
            points.append([point[1], point[0]])
        elif is_2d:
            points.append([point[0], point[1]])
        else:
            points.append([point[0], point[1], point[2]])
        t.append((current_date - init_date).seconds)
    return yupi.Trajectory(points=points, t=t)
